#! /usr/bin/env python

import os, re, sys, gzip
from itertools import product
from collections import defaultdict

# https://projects.parabola.nu/blacklist.git/plain/blacklist.txt

if len(sys.argv) != 4:
    print("use: python3 parabolize.py blacklist.txt path_to/arch-wiki/text/ path_to/write/")
    sys.exit(2)

blacklist = sys.argv[1]
load_index = os.path.join(sys.argv[2], 'index')
load_gz = os.path.join(sys.argv[2], 'arch-wiki.txt.gz')
save_index = os.path.join(sys.argv[3], 'index')
save_gz = os.path.join(sys.argv[3], 'arch-wiki.txt.gz')

def load_blacklist(path):
    okay_reasons = ['branding', 'technical']
    bad_reasons = ['semifree', 'nonfree', 'uses-nonfree', 'use-nonfree']
    whitelist = ['linux', 'mac']  # too many false positives
    for line in open(blacklist):
        pack,replace,*_,reason = line.split(':')
        if pack in whitelist:
            continue
        if pack == replace:
            continue
        if not replace:
            replace = re.sub('[a-zA-Z0-9]', 'X', pack)
        if any('['+r+']' in reason for r in bad_reasons):
            yield pack, replace

def remove_words(text_f, packages):
    "takes open file, returns line iterator"
    valid_delims = [' ', '\t', '\n', '@@b', '@@w', '"', "'", '/', '.', ',']
    hits = defaultdict(int)
    for line in text_f:
        line_lower = line.lower()
        for p, replace in packages:
            if p not in line_lower:
                continue
            if not any(d+p in line_lower for d in valid_delims):
                continue
            if not any(p+d in line_lower for d in valid_delims):
                continue
            print('detected %s in %s' % (p, line.strip()))
            cases = [p, p.upper(), p[0].upper() + p[1:]]
            for p2, prefix, suffix in product(cases, valid_delims, valid_delims):
                word = prefix + p2 + suffix
                if word not in line:
                    continue
                hits[p] += 1
                word2 = prefix + replace + suffix
                print('    replacing %s with %s' % (word.strip(), word2.strip()))
                line = line.replace(word, word2)
        yield line
    print('Stats:')
    hits = list((v,k) for k,v in hits.items())
    hits.sort()
    hits.reverse()
    for v,k in hits:
        print('\t', v, '\t', k)
    raise StopIteration

# remove long names first
packages = [(len(p), p, r) for p,r in load_blacklist(blacklist)]
packages.sort()
packages.reverse()
packages = list((p, r) for l,p,r in packages)

index2 = open(save_index, 'w')
index2.writelines(remove_words(open(load_index), packages))
index2.close()

gz2 = gzip.open(save_gz, 'wt', encoding='UTF-8')
gz2.writelines(remove_words(gzip.open(load_gz, 'rt', encoding='UTF-8'), packages))
gz2.close()

